@@ -67,7 +67,7 @@ class BrainFlowDevice(EEGDeviceInterface):
         self.eeg_channel_mapping = eeg_channel_mapping
         self.board_description = BoardShim.get_board_descr(board_id)
 
-        # Update channel names from config
+        # Update channel names from config.
         eeg_channel_idxs = sorted(list(eeg_channel_mapping.keys()))
         eeg_channel_names = [eeg_channel_mapping[idx] for idx in eeg_channel_idxs]
         self.board_description["eeg_names"] = ",".join(eeg_channel_names)
@@ -128,7 +128,6 @@ class DSI24Device(EEGDeviceInterface):
 
         # Data buffer and threading.
         self.data_buffer = []
-        self.marker_buffer = []
         self.timestamps_buffer = []
         self.is_streaming = False
         self.pull_thread = None
@@ -174,47 +173,29 @@ class DSI24Device(EEGDeviceInterface):
 
         # Try to get channel labels from the stream.
         self.channel_labels = full_info.get_channel_labels()
-        if self.channel_labels is None:
-            # If no labels in stream, use mapping or generate default.
-            if self.eeg_channel_mapping:
-                print(
-                    "No channel labels found in DSI-24 stream, use eeg_channel_mapping "
-                    + f"from config file: {self.eeg_channel_mapping}"
-                )
-                self.channel_labels = [
-                    self.eeg_channel_mapping.get(i, f"Ch{i+1}")
-                    for i in range(self.n_channels)
-                ]
-            else:
-                self.channel_labels = [f"Ch{i+1}" for i in range(self.n_channels)]
-                print(
-                    "No channel labels found in DSI-24 stream, "
-                    + "use default labels: {}"
-                )
-        else:  # self.channel_labels is not None
-            if self.eeg_channel_mapping:
-                print(
-                    "WARNING: Ignoring user-provided eeg_channel_mapping (from "
-                    + "config file), using channel labels from DSI-24 device instead: "
-                    + f"{self.channel_labels}"
-                )
-            else:
-                print(
-                    "Use EEG channel labels obtained from DSI-24 device: "
-                    + f"{self.channel_labels}"
-                )
 
-        print(f"Channel count: {self.n_channels}")
-        print(f"Sampling rate: {self.sampling_rate} Hz")
-        # print(f"Channel labels: {self.channel_labels}")
+        if self.eeg_channel_mapping is not None:
+            print(
+                "WARNING: eeg_channel_mapping from config yaml is ignored when using "
+                "DSI-24 device. Will get channel mapping from DSI-24 device."
+            )
 
-        # Create marker outlet for sending stimulus markers
+        # `self.channel_labels` is a list of strings, e.g. `["P3", "C3", "F3", "Fz",
+        # ...]`. Construct `eeg_channel_mapping` dictionary, e.g. `{0: 'P3', 1: 'C3', 2:
+        # 'F3', 3: 'Fz'}`. We assume that the order of channels in `self.channel_labels`
+        # corresponds to the rows of the array of EEG data obtained from the device.
+        # TODO: Check if that assumption holds.
+        self.eeg_channel_mapping = {}
+        for idx_channel, channel_label in enumerate(self.channel_labels):
+            self.eeg_channel_mapping[idx_channel] = channel_label
+
+        # Create marker outlet for sending stimulus markers.
         marker_info = self.StreamInfo(
             name="ExperimentMarkers",
             type="Markers",
             channel_count=1,
             nominal_srate=self.IRREGULAR_RATE,
-            channel_format="float32",
+            channel_format="float64",  # Could be lower precision, but simpler for compatibility with timestamps in numpy array.
             source_id="experiment_markers_" + str(hash(time.time())),
         )
         self.marker_outlet = self.StreamOutlet(marker_info)
@@ -258,59 +239,29 @@ class DSI24Device(EEGDeviceInterface):
                 if not self.is_streaming:
                     break
 
-    def get_board_data(self) -> np.ndarray:
+    def get_board_data(self) -> tuple[np.ndarray, np.ndarray]:
         """
-        Get accumulated data from the buffer and clear it. Returns data in format
-        compatible with BrainFlow (channels x samples).
+        Get accumulated data from the buffer and clear it. Returns EEG data and
+        corresponding timestamps.
         """
         with self.buffer_lock:
             if not self.data_buffer:
-                return np.array([]).reshape(self.n_channels + 1, 0)
+                # Return empty data and timestamps arrays.
+                return np.array([]).reshape(self.n_channels, 0), np.array([])
 
             # Convert buffered data to numpy array. Transpose to get channels x samples.
             data_array = np.array(self.data_buffer).T
             timestamps_array = np.array(self.timestamps_buffer)
 
-            # Add marker channel (last row) - initialize with zeros.
-            marker_channel = np.zeros(data_array.shape[1])
-
-            # Add any buffered markers to the marker channel. Match markers to data
-            # based on timestamps.
-            if self.marker_buffer:
-                for marker_val, marker_time in self.marker_buffer:
-                    # Find the closest timestamp in the data.
-                    if len(timestamps_array) > 0:
-                        closest_idx = np.argmin(np.abs(timestamps_array - marker_time))
-                        # Time difference between EEG data timestamps and marker
-                        # timestamp.
-                        time_delta = abs(timestamps_array[closest_idx] - marker_time)
-                        # Only add marker if it's reasonably close (within 20ms).
-                        if time_delta < 0.02:
-                            print(
-                                f"Adding marker {marker_val} with "
-                                + f"time delta {time_delta} s"
-                            )
-                            marker_channel[closest_idx] = marker_val
-                        else:
-                            print(
-                                f"WARNING: Skipping marker {marker_val} with "
-                                + f"time delta {time_delta} s"
-                            )
-
-            # Combine EEG data with marker channel
-            board_data = np.vstack([data_array, marker_channel.reshape(1, -1)])
-
-            # Clear buffers
+            # Clear buffers.
             self.data_buffer.clear()
             self.timestamps_buffer.clear()
-            self.marker_buffer.clear()
 
-            return board_data
+            return data_array, timestamps_array
 
     def insert_marker(self, marker: float):
         """
-        Insert a marker into the marker stream. Also buffer it locally for alignment
-        with EEG data.
+        Insert a marker into the LSL marker stream.
         """
         if self.marker_outlet:
             # Get current LSL timestamp.
@@ -319,11 +270,10 @@ class DSI24Device(EEGDeviceInterface):
             # Send marker through LSL outlet.
             self.marker_outlet.push_sample([marker], timestamp)
 
-            # Also store locally for alignment with get_board_data.
-            with self.buffer_lock:
-                self.marker_buffer.append((marker, timestamp))
-
-            print(f"Inserted marker {marker} at LSL time {timestamp}")
+            # The marker is also returned to be saved by the logging process.
+            # print(f"Inserted marker {marker} at LSL time {timestamp}")
+            return marker, timestamp
+        return None, None
 
     def release_session(self):
         """Clean up resources."""
@@ -337,7 +287,6 @@ class DSI24Device(EEGDeviceInterface):
         with self.buffer_lock:
             self.data_buffer.clear()
             self.timestamps_buffer.clear()
-            self.marker_buffer.clear()
 
         print("Released DSI-24 session")
 
@@ -346,24 +295,19 @@ class DSI24Device(EEGDeviceInterface):
         if not self.inlet:
             raise RuntimeError("Device not initialized. Call prepare_session() first.")
 
-        # Create board description similar to BrainFlow format
+        # Create board description similar to BrainFlow format.
         board_description = {
             "name": f"DSI-24 ({self.stream_info.name()})",
             "sampling_rate": self.sampling_rate,
-            "package_num_channel": 0,  # Not applicable for LSL
-            "timestamp_channel": -1,  # Using LSL timestamps
-            "marker_channel": self.n_channels,  # Last channel after EEG channels
-            "num_rows": self.n_channels + 1,  # EEG channels + marker channel
-            "eeg_channels": list(range(self.n_channels)),
             "eeg_names": ",".join(self.channel_labels),
         }
 
         return {
             "board_description": board_description,
             "sampling_rate": int(self.sampling_rate),
-            "eeg_channels": list(range(self.n_channels)),
-            "marker_channel": self.n_channels,  # Marker channel index
-            "n_channels_total": self.n_channels + 1,  # Include marker channel
+            "eeg_channels": None,  # For backward compatibility with Cyton
+            "marker_channel": None,  # For backward compatibility with Cyton
+            "n_channels_total": self.n_channels,
         }
 
 
