@@ -43,7 +43,7 @@ class EegExperimentConfig:
     eeg_device_address: Optional[str] = None  # Optional with default None
 
     # Use default_factory for mutable types
-    eeg_channel_mapping: Dict[int, str] = field(default_factory=dict)
+    eeg_channel_mapping: Optional[Dict[int, str]] = field(default_factory=dict)
 
     def __post_init__(self):
         """
